"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const directory_tree_1 = __importDefault(require("directory-tree"));
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const prettier_1 = __importDefault(require("prettier"));
const resolve_1 = require("./resolve");
const routes_1 = require("./template/routes");
function relativeTo(prefix, path) {
    const segments = path.split('/');
    const index = segments.findIndex(seg => seg === prefix);
    return segments.slice(index).join('/');
}
function toPathTree(directoryTree, prefix) {
    var _a;
    return {
        value: {
            path: path_1.isAbsolute(directoryTree.path)
                ? relativeTo(prefix, directoryTree.path)
                : directoryTree.path,
            type: directoryTree.type
        },
        children: (_a = directoryTree === null || directoryTree === void 0 ? void 0 : directoryTree.children) === null || _a === void 0 ? void 0 : _a.map(child => toPathTree(child, prefix))
    };
}
function generateRoutes({ folders, importPrefix = '@/assets/', dynamicImport = true, chunkNamePrefix = '' }) {
    const routeStrings = folders.map(folder => {
        const dirTree = directory_tree_1.default(folder, {
            extensions: /\.md$/,
            normalizePath: true
        });
        const pathTree = toPathTree(dirTree, path_1.basename(folder));
        const meta = resolve_1.resolveRoutePaths(pathTree, importPrefix, file => {
            return fs_1.default.readFileSync(path_1.join(path_1.dirname(folder), file), 'utf8');
        });
        return routes_1.createRoutes(meta, dynamicImport, chunkNamePrefix);
    });
    const rendererImport = `
  import Vue from 'vue'
  
  Vue.component('renderer', resolve => {
    setTimeout(() => resolve({
      render: createElement => createElement('router-view')
    }))
  })
  `;
    const imports = [rendererImport]
        .concat(routeStrings.map(rs => rs.imports))
        .join('\n');
    const routes = routeStrings.map(rs => rs.code).join(',\n');
    return prettier_1.default.format(`${imports}\n\nexport default [${routes}]`, {
        parser: 'babel',
        semi: false,
        singleQuote: true
    });
}
exports.generateRoutes = generateRoutes;
