"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const gray_matter_1 = __importDefault(require("gray-matter"));
const lodash_isempty_1 = __importDefault(require("lodash.isempty"));
const path_1 = __importDefault(require("path"));
const tree_1 = require("./tree");
function resolveRoutePaths(tree, importPrefix, readFile) {
    return sortByMeta(filterDirectoryMeta(tree_1.map(tree, (file, children) => {
        const segments = file.path.split('/');
        const meta = {
            name: pathToName(segments),
            path: pathToRoute(segments, 0, false),
            pathSegments: toActualPath(segments),
            component: null,
            specifier: pathToSpecifier(segments)
        };
        if (file.type === 'file') {
            const content = readFile(file.path);
            const { data } = gray_matter_1.default(content);
            if (!lodash_isempty_1.default(data)) {
                meta.routeMeta = data;
            }
            meta.component = importPrefix + file.path;
        }
        if (file.type === 'directory' && (children === null || children === void 0 ? void 0 : children.length)) {
            const metaFile = children.find(child => child.value.type === 'file' &&
                basename(child.value.path) === '__meta__');
            if (metaFile !== undefined) {
                const content = readFile(metaFile.value.path);
                const { data } = gray_matter_1.default(content);
                if (!lodash_isempty_1.default(data)) {
                    meta.routeMeta = data;
                }
            }
        }
        return meta;
    })));
}
exports.resolveRoutePaths = resolveRoutePaths;
function sortByMeta(tree) {
    return tree_1.sort(tree, (a, b) => {
        const routeMeta1 = a.value.routeMeta;
        const routeMeta2 = b.value.routeMeta;
        if (hasOrder(routeMeta1)) {
            if (hasOrder(routeMeta2)) {
                return routeMeta1.order - routeMeta2.order;
            }
            return -1;
        }
        if (hasOrder(routeMeta2)) {
            return 1;
        }
        return 0;
    });
}
function hasOrder(obj) {
    return typeof obj === 'object' && obj !== null && obj.hasOwnProperty('order');
}
function filterDirectoryMeta(tree) {
    return tree_1.filter(tree, meta => { var _a; return !((_a = meta.component) === null || _a === void 0 ? void 0 : _a.endsWith('__meta__.md')); });
}
function isOmittable(segment) {
    return segment === 'index';
}
/**
 * - Remove `.md` from the last path
 * - Omit if the last segment is `index`
 */
function toActualPath(segments) {
    const lastIndex = segments.length - 1;
    const last = basename(segments[lastIndex]);
    const segmentsWithoutLast = segments.slice(0, -1);
    return isOmittable(last)
        ? segmentsWithoutLast
        : segmentsWithoutLast.concat(last);
}
function pathToName(segments) {
    const last = segments[segments.length - 1];
    return segments
        .slice(0, -1)
        .concat(basename(last))
        .join('-');
}
function pathToSpecifier(segments) {
    const name = pathToName(segments);
    const replaced = name
        .replace(/(^|[^a-zA-Z])([a-zA-Z])/g, (_, a, b) => {
        return a + b.toUpperCase();
    })
        .replace(/[^a-zA-Z0-9]/g, '');
    return /^\d/.test(replaced) ? '_' + replaced : replaced;
}
function pathToRoute(segments, parentDepth, nested) {
    const prefix = nested || parentDepth > 0 ? '' : '/';
    return (prefix +
        toActualPath(segments)
            .slice(parentDepth)
            .join('/'));
}
function basename(filename) {
    return path_1.default.basename(filename, '.md');
}
