"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tree_1 = require("../tree");
function createChildrenRoute(children) {
    return `children: [${children.map(createRoute).join(',')}],`;
}
function createRoute(meta) {
    var _a, _b, _c, _d;
    const children = !meta.children ? '' : createChildrenRoute(meta.children);
    // If default child exists, the route should not have a name.
    const routeName = ((_a = meta.children) === null || _a === void 0 ? void 0 : _a.some(m => m.value.path === '')) ? ''
        : `name: '${meta.value.name}',`;
    const routeMeta = meta.value.routeMeta
        ? `meta: ${JSON.stringify(meta.value.routeMeta, null, 2)},`
        : '';
    const redirect = (_c = (_b = meta.children) === null || _b === void 0 ? void 0 : _b.find(child => child.value.specifier.endsWith('Index'))) !== null && _c !== void 0 ? _c : (_d = meta.children) === null || _d === void 0 ? void 0 : _d[0];
    const specifier = meta.value.component
        ? `component: ${meta.value.specifier},`
        : `component: Vue.component('renderer'),
    redirect: { name: '${redirect.value.name}' },
    `;
    return `
  {
    ${routeName}
    path: '${meta.value.path}',
    ${specifier}${routeMeta}${children}
  }
  `;
}
function createImport(meta, dynamic, chunkNamePrefix) {
    if (meta.component === null) {
        return '';
    }
    return dynamic
        ? `function ${meta.specifier}() { return import(/* webpackChunkName: "${chunkNamePrefix}${meta.name}" */ '${meta.component}') }`
        : `import ${meta.specifier} from '${meta.component}'`;
}
function createRoutes(meta, dynamic, chunkNamePrefix) {
    const imports = tree_1.flatMap(meta, node => createImport(node, dynamic, chunkNamePrefix)).join('\n');
    const code = createRoute(meta);
    return { imports, code };
}
exports.createRoutes = createRoutes;
