"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function forEach(tree, fn) {
    const { value, children } = tree;
    fn(value);
    if (!children) {
        return;
    }
    children.forEach(child => {
        forEach(child, fn);
    });
}
exports.forEach = forEach;
function map(tree, fn) {
    const { value, children } = tree;
    if (!children) {
        return { value: fn(value) };
    }
    return {
        value: fn(value, children),
        children: children.map(child => map(child, fn))
    };
}
exports.map = map;
function flatten(tree) {
    function flattenRec(acc, node) {
        acc.push(node.value);
        if (node.children) {
            acc.concat(...node.children.map(child => flattenRec(acc, child)));
        }
        return acc;
    }
    return flattenRec([], tree);
}
exports.flatten = flatten;
function flatMap(tree, fn) {
    return flatten(map(tree, fn));
}
exports.flatMap = flatMap;
function sort(tree, fn) {
    const { value, children } = tree;
    if (!children) {
        return tree;
    }
    return {
        value,
        children: [...children].map(child => sort(child, fn)).sort(fn)
    };
}
exports.sort = sort;
function filter(tree, fn) {
    if (!fn(tree.value)) {
        return null;
    }
    if (!tree.children) {
        return tree;
    }
    return {
        value: tree.value,
        children: tree.children
            .map(child => filter(child, fn))
            .filter(v => v !== null)
    };
}
exports.filter = filter;
